@@ -619,7 +619,7 @@ int aim_conn_completeconnect(aim_session_t *sess, aim_conn_t *conn)
 	} 
 
 	if (FD_ISSET(conn->fd, &fds) || FD_ISSET(conn->fd, &wfds)) {
-		int len = sizeof(error);
+		unsigned int len = sizeof(error);
 
 		if (getsockopt(conn->fd, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
 			error = errno;
