@@ -36,8 +36,7 @@
 #include <arpa/inet.h>
 #include <unistd.h>
 #else
-#include <winsock.h>
-int inet_aton(char *host, struct in_addr *addr) { addr->S_un.S_addr = inet_addr(host); return addr->S_un.S_addr != INADDR_NONE; }
+#include "sock.h"
 #define ETIMEDOUT WSAETIMEDOUT
 #define EINPROGRESS WSAEINPROGRESS
 #endif
@@ -107,11 +106,65 @@ static void gaim_io_destroy(gpointer data)
 	g_free(data);
 }
 
+#ifdef PROXYPROFILER
+struct proxyprofiler
+{
+	GaimInputFunction function;
+	gpointer data;
+	
+	int count;
+	
+	struct proxyprofiler *next;
+} *pp = NULL;
+
+void proxyprofiler_dump()
+{
+	struct proxyprofiler *l;
+	char s[128];
+	FILE *fp;
+	
+	sprintf( s, "proxyprofiler.%d", (int) getpid() );
+	fp = fopen( s, "w" );
+	
+	fprintf( fp, "%-18s  %-18s  %10s\n", "Function", "Data", "Count" );
+	for( l = pp; l; l = l->next )
+		fprintf( fp, "0x%-16x  0x%-16x  %10d\n", (int) l->function, (int) l->data, l->count );
+	
+	fclose( fp );
+}
+#endif
+
 static gboolean gaim_io_invoke(GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	GaimIOClosure *closure = data;
 	GaimInputCondition gaim_cond = 0;
 
+#ifdef PROXYPROFILER
+	struct proxyprofiler *l;
+	
+	for( l = pp; l; l = l->next )
+	{
+		if( closure->function == l->function && closure->data == l->data )
+			break;
+	}
+	if( l )
+	{
+		l->count ++;
+	}
+	else
+	{
+		l = g_new0( struct proxyprofiler, 1 );
+		l->function = closure->function;
+		l->data = closure->data;
+		l->count = 1;
+		
+		l->next = pp;
+		pp = l;
+	}
+#endif
+	
+	count_io_event(source, "proxy");
+	
 	if (condition & GAIM_READ_COND)
 		gaim_cond |= GAIM_INPUT_READ;
 	if (condition & GAIM_WRITE_COND)
@@ -592,7 +645,7 @@ int proxy_connect(char *host, int port, GaimInputFunction func, gpointer data)
 {
 	struct PHB *phb;
 	
-	if (!host || !port || (port == -1) || !func) {
+	if (!host || !port || (port == -1) || !func || strlen(host) > 128) {
 		return -1;
 	}
 	
